SystemOrganization addCategory: #'OpenAL-Pools'!
SystemOrganization addCategory: #'OpenAL-Base'!
SystemOrganization addCategory: #'OpenAL-Objects'!
SystemOrganization addCategory: #'OpenAL-Misc'!
SystemOrganization addCategory: #'OpenAL-Harness'!
SystemOrganization addCategory: #'OpenAL-Chat'!
SystemOrganization addCategory: #'OpenAL-Tests'!

!TFrame methodsFor: '*OpenAL' stamp: 'jcg 7/25/2006 00:11'!
openAL
	^ OpenAL singleton! !

!SoundCodec methodsFor: '*OpenAL' stamp: 'pbm 3/15/2006 14:52'!
decodeCompressedDataNoReset: aByteArray
	"Decode the entirety of the given encoded data buffer with this codec. Answer a monophonic SoundBuffer containing the uncompressed samples."

	| frameCount result increments |
	frameCount := self frameCount: aByteArray.
	result := SoundBuffer newMonoSampleCount: frameCount * self samplesPerFrame.
	"self reset."
	increments := self decodeFrames: frameCount from: aByteArray at: 1 into: result at: 1.
	((increments first = aByteArray size) and: [increments last = result size]) ifFalse: [
		self error: 'implementation problem; increment sizes should match buffer sizes'].
	^ result
! !

!SoundCodec methodsFor: '*OpenAL' stamp: 'pbm 3/15/2006 15:37'!
encodeSoundBufferNoReset: aSoundBuffer
	"Encode the entirety of the given monophonic SoundBuffer with this codec. Answer a ByteArray containing the compressed sound data."

	| codeFrameSize frameSize fullFrameCount lastFrameSamples result increments finalFrame i lastIncs |
	frameSize := self samplesPerFrame.
	fullFrameCount := aSoundBuffer monoSampleCount // frameSize.
	lastFrameSamples := aSoundBuffer monoSampleCount - (fullFrameCount * frameSize).
	codeFrameSize := self bytesPerEncodedFrame.
	codeFrameSize = 0 ifTrue:
		["Allow room for 1 byte per sample for variable-length compression"
		codeFrameSize := frameSize].
	lastFrameSamples > 0
		ifTrue: [result := ByteArray new: (fullFrameCount + 1) * codeFrameSize]
		ifFalse: [result := ByteArray new: fullFrameCount * codeFrameSize].
"	self reset."
	increments := self encodeFrames: fullFrameCount from: aSoundBuffer at: 1 into: result at: 1.
	lastFrameSamples > 0 ifTrue: [
		finalFrame := SoundBuffer newMonoSampleCount: frameSize.
		i := fullFrameCount * frameSize.
		1 to: lastFrameSamples do: [:j |
			finalFrame at: j put: (aSoundBuffer at: (i := i + 1))].
		lastIncs := self encodeFrames: 1 from: finalFrame at: 1 into: result at: 1 + increments second.
		increments := Array with: increments first + lastIncs first
							with: increments second + lastIncs second].
	increments second < result size
		ifTrue: [^ result copyFrom: 1 to: increments second]
		ifFalse: [^ result]
! !

!OrderedCollection methodsFor: '*OpenAL' stamp: 'pbm 3/23/2006 14:26'!
removeAll
	"Remove all elements of the receiver"
	firstIndex to: lastIndex do: [:index | array at: index put: nil].
	lastIndex := firstIndex - 1! !

SoundRecorder subclass: #TVoiceRecorder
	instanceVariableNames: 'frameTime testFeedback volumeAverage volumeBuffers streamer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenAL-Chat'!

!TVoiceRecorder commentStamp: '<historical>' prior: 0!
TVoiceRecorder new!

!TVoiceRecorder methodsFor: 'initialization' stamp: 'das 4/11/2003 16:08'!
addLocalFeedback
	testFeedback := true.
! !

!TVoiceRecorder methodsFor: 'private' stamp: 'ar 3/12/2002 22:01'!
allocateBuffer
	"Allocate a new buffer and reset nextIndex."
	| bufSize |
	"Multiple of samplesPerFrame that is approx. bufferTime long"
	bufSize := (self bufferSize truncateTo: self samplesPerFrame) max: self samplesPerFrame.
	(currentBuffer == nil or:[currentBuffer monoSampleCount ~= bufSize]) ifTrue:[
		currentBuffer := SoundBuffer newMonoSampleCount: bufSize.
	].
	nextIndex := 1.
! !

!TVoiceRecorder methodsFor: 'private' stamp: 'das 4/18/2003 01:33'!
bufferSize
	"Buffer size should be small enough to
		* fit into a UDP packet even if worst-case compressed
		* be small enough not to introduce addl. latency (even on 11kHz)
	"
	^1600

! !

!TVoiceRecorder methodsFor: 'private' stamp: 'ar 3/12/2002 21:28'!
emitBuffer: buffer
	"Do nothing here - all work is done in #sendBuffer:"! !

!TVoiceRecorder methodsFor: 'initialization' stamp: 'das 4/15/2003 11:18'!
initialize
	super initialize.
	codec := GSMCodec new.
	volumeBuffers := 1.
	volumeAverage := 0.
"	self addLocalFeedback."
! !

!TVoiceRecorder methodsFor: 'recording controls' stamp: 'ar 5/20/2002 16:19'!
isRecording
	^recordProcess notNil! !

!TVoiceRecorder methodsFor: 'accessing' stamp: 'das 4/18/2003 00:41'!
noiseThreshold
	"Heuristic value for now..."
	^1000! !

!TVoiceRecorder methodsFor: 'private' stamp: 'ar 6/15/2002 20:04'!
recordLoop
	"Record process loop that records samples."

	| n sampleCount repeat hasSound |
	n := 0.
	repeat := 5.
	[true] whileTrue: [
		n = 0 ifTrue: [bufferAvailableSema wait].
		n := self primRecordSamplesInto: currentBuffer startingAt: nextIndex.
		nextIndex := nextIndex + n.
		stereo
			ifTrue: [sampleCount := currentBuffer stereoSampleCount]
			ifFalse: [sampleCount := currentBuffer monoSampleCount].
		nextIndex > sampleCount ifTrue: [
			hasSound := self sendBuffer: currentBuffer ifSilent: (repeat > 0).
			repeat := hasSound ifTrue:[5] ifFalse:[repeat - 1].
			self allocateBuffer.
		].
	].
! !

!TVoiceRecorder methodsFor: 'private' stamp: 'pbm 3/16/2006 12:51'!
sendBuffer: buffer ifSilent: sendAnyways
	"Send the shared buffer"
	| data nowFrameTime hasSound useCodec|
	nowFrameTime := frameTime.

true ifTrue:[	useCodec := codec.] ifFalse:[	useCodec := nil.].
	stereo 
		ifTrue:[frameTime := frameTime + (1000.0 * buffer stereoSampleCount / samplingRate)]
		ifFalse:[frameTime := frameTime + (1000.0 * buffer monoSampleCount / samplingRate)].
	self meterFrom: 1 count: buffer monoSampleCount in: buffer.
	volumeBuffers > 10 ifTrue:[volumeBuffers := 0. volumeAverage := 0].
	volumeAverage := volumeAverage + meterLevel.
	volumeBuffers := volumeBuffers+1.
	hasSound := meterLevel >  self noiseThreshold.
	(hasSound or:[sendAnyways]) ifFalse:[^hasSound].
	useCodec 
		ifNil:[data := buffer asByteArray]
		ifNotNil:[data := useCodec encodeSoundBufferNoReset: buffer].

	streamer ifNotNil:[
		streamer playSound:data at:nowFrameTime samplingRate: samplingRate codec:(useCodec ifNotNil:[useCodec class]).
	].

	^hasSound! !

!TVoiceRecorder methodsFor: 'recording controls' stamp: 'pbm 3/16/2006 13:26'!
startRecording
	frameTime := 0.
	super startRecording.
	streamer start.
! !

!TVoiceRecorder methodsFor: 'accessing' stamp: 'ar 6/17/2002 00:09'!
statusString
	self isRecording ifFalse:[^''].
	^'Chat Volume: ', self volumeAverage printString, '%'! !

!TVoiceRecorder methodsFor: 'recording controls' stamp: 'pbm 3/16/2006 13:04'!
stopRecording
	super stopRecording.
	streamer stop.! !

!TVoiceRecorder methodsFor: 'accessing' stamp: 'pbm 3/16/2006 12:51'!
streamer
	^ streamer! !

!TVoiceRecorder methodsFor: 'accessing' stamp: 'pbm 3/16/2006 12:51'!
streamer: aTSoundStreamer
	streamer := aTSoundStreamer.! !

!TVoiceRecorder methodsFor: 'accessing' stamp: 'ar 6/16/2002 23:58'!
volumeAverage
	^(volumeAverage max: 0) * 100 / volumeBuffers // 32768! !

Object subclass: #Broadcaster
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenAL-Misc'!

!Broadcaster commentStamp: '<historical>' prior: 0!
Broadcasts messages to all objects that have registered interest.  Uses the Scripting Extensions, as well as the DNU mechanism.

*** I had to change this to use events. See TOpenALResource for details.!

!Broadcaster methodsFor: 'DNU' stamp: 'pbm 3/8/2006 17:04'!
doesNotUnderstand: aMessage
	"Broadcast message to everyone who cares when I broadcast something." 
	self 	
		triggerEvent: #broadcastMessage 
		withArguments: {aMessage selector. aMessage arguments}! !

Object subclass: #FallbackOpenAL
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenAL-Base'!

!FallbackOpenAL commentStamp: 'jcg 8/19/2006 10:41' prior: 0!
This class is used if an OpenAL context cannot be created.!

!FallbackOpenAL methodsFor: 'error handling' stamp: 'hrs 5/25/2006 15:13'!
doesNotUnderstand: aMessage
	! !

!FallbackOpenAL methodsFor: 'accessing' stamp: 'hrs 5/25/2006 15:10'!
ifNotNilDo: aBlock! !

!FallbackOpenAL methodsFor: 'testing' stamp: 'hrs 5/25/2006 14:58'!
isCurrent
	^true! !

!FallbackOpenAL methodsFor: 'testing' stamp: 'hrs 5/25/2006 15:13'!
isPlaying: ignored
	^true! !

Object subclass: #TSoundQueue
	instanceVariableNames: 'buffers mutex codec frameTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenAL-Chat'!

!TSoundQueue commentStamp: '<historical>' prior: 0!
Queues up sound data used for voice chat.!

!TSoundQueue methodsFor: 'accessing' stamp: 'pbm 3/22/2006 15:54'!
addSoundData: data at: deltaMSecs codec: codecClass
	| buffer |

	(deltaMSecs <= frameTime and:[ buffers size > 0]) ifTrue:[ ^0]. "you are late..."

	codecClass 
		ifNil:[buffer := SoundBuffer fromByteArray: data]
		ifNotNil:[
			codec class ~= codecClass ifTrue:[ codec := codecClass new.].
			buffer := codec decodeCompressedDataNoReset: data.
		].

	mutex critical:[ buffers add: (deltaMSecs -> buffer) ].
	buffers size printString displayAt: 0@60.

	self signal: #newSound.

	^ buffer	
! !

!TSoundQueue methodsFor: 'accessing' stamp: 'pbm 3/23/2006 10:19'!
flush
	buffers isEmpty ifFalse: [ buffers removeAll ].! !

!TSoundQueue methodsFor: 'initialize' stamp: 'pbm 3/15/2006 14:57'!
initialize

	super initialize.
	frameTime := 0.
	buffers := SortedCollection sortBlock:[:a1 :a2 | a1 key <= a2 key].
	mutex := Semaphore forMutualExclusion.! !

!TSoundQueue methodsFor: 'testing' stamp: 'pbm 3/22/2006 16:20'!
isEmpty
	^ buffers isEmpty! !

!TSoundQueue methodsFor: 'accessing' stamp: 'pbm 3/22/2006 17:10'!
nextSound
	| next |

	next := nil.
	mutex critical:[buffers size = 0 ifFalse:[next := buffers removeFirst]].

	next ifNotNil:[
		frameTime := next key.
		next := next value.
		"next := SampledSound samples: next samplingRate: 11025."
	].

	^ next.! !

!TSoundQueue methodsFor: 'accessing' stamp: 'pbm 3/22/2006 10:22'!
size
	^ buffers size! !

SharedPool subclass: #AlcConstants
	instanceVariableNames: ''
	classVariableNames: 'AlcAllAttributes AlcAttributesSize AlcCaptureDefaultDeviceSpecifier AlcCaptureDeviceSpecifier AlcCaptureSamples AlcDefaultDeviceSpecifier AlcDeviceSpecifier AlcExtensions AlcFalse AlcFrequency AlcInvalid AlcInvalidContext AlcInvalidDevice AlcInvalidEnum AlcInvalidValue AlcMajorVersion AlcMinorVersion AlcMonoSources AlcNoError AlcOutOfMemory AlcRefresh AlcStereoSources AlcSync AlcTrue'
	poolDictionaries: ''
	category: 'OpenAL-Pools'!

!AlcConstants class methodsFor: 'as yet unclassified' stamp: 'pbm 3/29/2006 17:05'!
initialize

	AlcInvalid := 0.
	AlcFalse := 0.
	AlcTrue := 1.
	AlcNoError := AlcFalse.

	AlcFrequency := 16r1007.
	AlcRefresh := 16r1008.
	AlcSync := 16r1009.
	AlcMonoSources := 16r1010.
	AlcStereoSources := 16r1011.

	AlcInvalidDevice := 16rA001.
	AlcInvalidContext := 16rA002.
	AlcInvalidEnum := 16rA003.
	AlcInvalidValue := 16rA004.
	AlcOutOfMemory := 16rA005.

	AlcDefaultDeviceSpecifier := 16r1004.
	AlcDeviceSpecifier := 16r1005.
	AlcExtensions := 16r1006.
	AlcMajorVersion := 16r1000.
	AlcMinorVersion := 16r1001.
	AlcAttributesSize := 16r1002.
	AlcAllAttributes := 16r1003.

	AlcCaptureDeviceSpecifier := 16r310.
	AlcCaptureDefaultDeviceSpecifier := 16r311.
	AlcCaptureSamples := 16r312.

	"check the class pool"
	self classPool keysAndValuesDo:[:key :val|
		val ifNil:[self error: key printString,' undefined'].
	].! !

SharedPool subclass: #OpenALConstants
	instanceVariableNames: ''
	classVariableNames: 'ALBits ALBuffer ALBuffersProcessed ALBuffersQueued ALChannelMask ALChannels ALConeInnerAngle ALConeOuterAngle ALConeOuterGain ALData ALDirection ALDistanceModel ALDistanceScale ALDopplerFactor ALDopplerVelocity ALExtentions ALFalse ALFormatMono16 ALFormatMono8 ALFormatStereo16 ALFormatStereo8 ALFrequency ALGain ALIllegalCommand ALIllegalEnum ALInitial ALInvalid ALInvalidEnum ALInvalidName ALInvalidOperation ALInvalidValue ALInverseDistance ALInverseDistanceClamped ALLooping ALMaxDistance ALMaxGain ALMinGain ALNoError ALNone ALOrientation ALOutOfMemory ALPaused ALPending ALPitch ALPlaying ALPosition ALProcessed ALReferenceDistance ALRenderer ALRolloffFactor ALSize ALSourceAbsolute ALSourceRelative ALSourceState ALSourceType ALSpeedOfSound ALStopped ALStreaming ALTrue ALUnused ALVelocity ALVendor ALVersion'
	poolDictionaries: ''
	category: 'OpenAL-Pools'!

Object subclass: #OpenAL
	instanceVariableNames: 'handle nameManager'
	classVariableNames: 'Default InstalledOpenALLibrary'
	poolDictionaries: 'OpenALConstants AlcConstants'
	category: 'OpenAL-Base'!

!OpenAL commentStamp: '<historical>' prior: 0!
This is the OpenAL object containing methods for all of the primitive C functions in the OpenAL plugin.!

OpenAL subclass: #OpenALMacOSX
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenAL-Base'!

!OpenALMacOSX methodsFor: 'initialize' stamp: 'pbm 3/30/2006 16:39'!
openALLibraryName
	^'OpenAL.framework'! !

OpenAL subclass: #OpenALUnix
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenAL-Base'!

!OpenALUnix commentStamp: 'mpm 4/4/2006 12:56' prior: 0!
for some reason we had to overide the OpenAL initialize method in this subclass
to get Linux to  get the correct privateInstallLibrary. Thid was not necessary
for macintosh or Windows... weird. We'll fix this nasty hack once we figure out
what is going on... - mpm"
!

!OpenALUnix methodsFor: 'initialize' stamp: 'etw 4/10/2006 15:18'!
initialize

   "@@@@@ FIXME @@@@@
    for some reason, the usual method of getting the correct privateInstallLibrary
    is not working under Linux, so we resort to a hack and override the initialize method
    in the OpenALUnix subclass. Nasty hack, but it works... - mpm"

	self privateInstallLibrary: 'openal'.
	nameManager := OpenALNameManager new initialize: self.
! !

!OpenALUnix methodsFor: 'initialize' stamp: 'etw 4/4/2006 10:05'!
openALLibraryName
	^'openal'! !

OpenAL subclass: #OpenALWin32
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenAL-Base'!

!OpenALWin32 methodsFor: 'initialize' stamp: 'pbm 3/31/2006 12:58'!
openALLibraryName
	^'OpenAL32.dll'! !

!OpenAL class methodsFor: 'instance creation' stamp: 'jcg 7/25/2006 00:46'!
destroy
	Default ifNotNil: 
		[Default destroy.
		Default := nil]! !

!OpenAL class methodsFor: 'instance creation' stamp: 'hrs 8/26/2006 16:23'!
doWithoutErrors: aBlock
	^aBlock
		on: Error 
		do: 
			[:err |
			Transcript show: 'OpenAL error: ', err; cr.
			^FallbackOpenAL new].
! !

!OpenAL class methodsFor: 'class initialization' stamp: 'pbm 3/30/2006 16:47'!
initialize
	"OpenAL initialize"
	Smalltalk addToStartUpList: self.! !

!OpenAL class methodsFor: 'instance creation' stamp: 'jcg 8/27/2006 23:13'!
new
	| cls |
	Default ifNotNil: [^ Default].
	Smalltalk platformName = 'Win32' ifTrue: [cls := OpenALWin32].
	Smalltalk platformName = 'unix' ifTrue:[cls := OpenALUnix].
	Smalltalk platformName = 'Mac OS' ifTrue:[cls := OpenALMacOSX].
	^self doWithoutErrors: [Default := cls basicNew initialize].
! !

!OpenAL class methodsFor: 'instance creation' stamp: 'jcg 7/25/2006 00:47'!
singleton
	^ self new! !

!OpenAL class methodsFor: 'class initialization' stamp: 'jcg 7/25/2006 00:46'!
startUp
	"Flush the installed OpenALLib"
	InstalledOpenALLibrary := nil.
	Default := nil.
! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:40'!
alBufferData(buffer, format, data, size, freq)
	"ALvoid alBufferData(ALuint buffer,ALenum format,ALvoid *data,ALsizei size,ALsizei freq);"
	<apicall: void 'alBufferData' (ulong ulong void* long long) module: 'OpenAL32.dll'>

	^ self externalCallFailed
! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/30/2006 14:31'!
alBufferData: buffer with: format with: data with: size with: freq
	"ALvoid alBufferData(ALuint buffer,ALenum format,ALvoid *data,ALsizei size,ALsizei freq);"
	<apicall: void 'alBufferData' (ulong ulong void* long long) module: 'OpenAL32.dll'>

	^ self externalCallFailed
	! !

!OpenAL methodsFor: 'Alc API' stamp: 'pbm 3/31/2006 16:12'!
alcCloseDevice: device
	"void alcCloseDevice( ALCdevice *dev );"
	<apicall: void 'alcCloseDevice' (void*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Alc API' stamp: 'pbm 3/31/2006 16:12'!
alcCreateContext: device with: list
	"void * alcCreateContext( ALCdevice *dev, ALint* attrlist );"
	<apicall: void* 'alcCreateContext' (void* long*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Alc API' stamp: 'pbm 3/31/2006 11:22'!
alcDestroyContext: context
	"ALCenum alcDestroyContext( ALvoid *alcHandle );"
	<apicall: ulong 'alcDestroyContext' (void*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Alc API' stamp: 'pbm 3/31/2006 11:22'!
alcGetContextsDevice: context
	"ALCdevice* alcGetContextsDevice( ALvoid *alcHandle );"
	<apicall: void* 'alcGetContextsDevice' (void*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Alc API' stamp: 'pbm 3/29/2006 15:57'!
alcGetCurrentContext
	"void * alcGetCurrentContext( ALvoid );"
	<apicall: void* 'alcGetCurrentContext' (void) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Alc API' stamp: 'pbm 4/5/2006 13:15'!
alcGetEnumValue: device with: name
	"ALenum alcGetEnumValue(ALCdevice *device, ALubyte *enumName);"
	<apicall: ulong 'alcGetEnumValue' (void* byte*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Alc API' stamp: 'pbm 4/5/2006 13:15'!
alcGetError
	"ALenum alcGetError(ALVoid);"
	<apicall: ulong 'alcGetError' (void) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Alc API' stamp: 'pbm 3/31/2006 16:13'!
alcGetIntegerv: device with: token with: size with: dest
	"ALvoid alcGetIntegerv(ALCdevice *device, ALenum token, ALsizei size, ALint *dest);"
	<apicall: void 'alcGetIntegerv' (void* ulong long long*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Alc API' stamp: 'pbm 3/31/2006 16:13'!
alcGetProcAddress: device with: name
	"ALvoid * alcGetProcAddress(ALCdevice *device, ALubyte *funcName);"
	<apicall: void* 'alcGetProcAddress' (void* byte*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Alc API' stamp: 'pbm 3/31/2006 16:12'!
alcGetString: device with: token
	"ALubyte * alcGetString(ALCdevice *device, ALenum token);"
	<apicall: byte* 'alcGetString' (void* ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Alc API' stamp: 'pbm 3/31/2006 16:14'!
alcIsExtensionPresent: device with: name
	"ALboolean alcIsExtensionPresent(ALCdevice *device, ALubyte *extName);"
	<apicall: bool 'alcIsExtensionPresent' (void* byte*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Alc API' stamp: 'pbm 3/31/2006 11:22'!
alcMakeContextCurrent: context
	"ALCenum alcMakeContextCurrent( ALvoid *alcHandle );"
	<apicall: ulong 'alcMakeContextCurrent' (void*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Alc API' stamp: 'pbm 3/31/2006 16:14'!
alcOpenDevice: string
	"ALCdevice *alcOpenDevice( const ALubyte *tokstr );"
	<apicall: void* 'alcOpenDevice' (byte*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Alc API' stamp: 'pbm 3/31/2006 11:22'!
alcProcessContext: context
	"void alcProcessContext( ALvoid *alcHandle );"
	<apicall: void 'alcProcessContext' (void*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Alc API' stamp: 'pbm 3/31/2006 11:22'!
alcSuspendContext: context
	"void alcSuspendContext( ALvoid *alcHandle );"
	<apicall: void 'alcSuspendContext' (void*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:40'!
alDeleteBuffers(n, buffers)
	"ALvoid alDeleteBuffers(ALsizei n,ALuint *buffers)"
	<apicall: void 'alDeleteBuffers' (long ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 13:52'!
alDeleteBuffers: n with: buffers
	"ALvoid alDeleteBuffers(ALsizei n,ALuint *buffers)"
	<apicall: void 'alDeleteBuffers' (long ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/29/2006 13:54'!
alDeleteBuffer: buffer

	self alDeleteBuffers: 1 with: (WordArray with: buffer).! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:40'!
alDeleteSources(n, sources)
	"ALvoid alDeleteSources(ALsizei n,ALuint *sources)"
	<apicall: void 'alDeleteSources' (long ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 13:52'!
alDeleteSources: n with: sources
	"ALvoid alDeleteSources(ALsizei n,ALuint *sources)"
	<apicall: void 'alDeleteSources' (long ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/29/2006 13:55'!
alDeleteSource: source

	self alDeleteSources: 1 with: (WordArray with: source).! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:40'!
alDisable(capability)
	"ALvoid alDisable(ALenum capability);"
	<apicall: void 'alDisable' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:13'!
alDisable: capability
	"ALvoid alDisable(ALenum capability);"
	<apicall: void 'alDisable' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:41'!
alDistanceModel(value)
	"ALvoid alDistanceModel( ALenum value );"
	<apicall: void 'alDistanceModel' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:14'!
alDistanceModel: value
	"ALvoid alDistanceModel( ALenum value );"
	<apicall: void 'alDistanceModel' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:41'!
alDopplerFactor(value)
	"ALvoid alDopplerFactor( ALfloat value );"
	<apicall: void 'alDopplerFactor' (float) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:14'!
alDopplerFactor: value
	"ALvoid alDopplerFactor( ALfloat value );"
	<apicall: void 'alDopplerFactor' (float) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:41'!
alDopplerVelocity(value)
	"ALvoid alDopplerVelocity( ALfloat value );"
	<apicall: void 'alDopplerVelocity' (float) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:14'!
alDopplerVelocity: value
	"ALvoid alDopplerVelocity( ALfloat value );"
	<apicall: void 'alDopplerVelocity' (float) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:41'!
alEnable(capability)
	"ALvoid alEnable(ALenum capability);"
	<apicall: void 'alEnable' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:15'!
alEnable: capability
	"ALvoid alEnable(ALenum capability);"
	<apicall: void 'alEnable' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/22/2006 16:03'!
alGenBuffer

	| holder |
	holder := WordArray new: 1.
	self alGenBuffers: 1 with: holder.

	^ holder at: 1! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:41'!
alGenBuffers(n, buffers)
	"ALvoid alGenBuffers(ALsizei n,ALuint *buffers)"
	<apicall: void 'alGenBuffers' (long ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/22/2006 16:03'!
alGenBuffers: n

	| holder |
	holder := WordArray new: n.
	self alGenBuffers: n with: holder.

	^ holder! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 13:49'!
alGenBuffers: n with: buffers
	"ALvoid alGenBuffers(ALsizei n,ALuint *buffers)"
	<apicall: void 'alGenBuffers' (long ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/29/2006 15:13'!
alGenSource

	| holder |
	holder := WordArray new: 1.
	self alGenSources: 1 with: holder.

	^holder at: 1! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:41'!
alGenSources(n, sources)
	"ALvoid alGenSources(ALsizei n,ALuint *sources)"
	<apicall: void 'alGenSources' (long ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/29/2006 15:14'!
alGenSources: n

	| holder |
	holder := WordArray new: n.
	self alGenSources: n with: holder.

	^holder! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 13:50'!
alGenSources: n with: sources
	"ALvoid alGenSources(ALsizei n,ALuint *sources)"
	<apicall: void 'alGenSources' (long ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:42'!
alGetBooleanv(pname, data)
	"ALvoid alGetBooleanv(ALenum pname,ALboolean *data);"
	<apicall: void 'alGetBooleanv' (ulong ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:21'!
alGetBooleanv: pname with: data
	"ALvoid alGetBooleanv(ALenum pname,ALboolean *data);"
	<apicall: void 'alGetBooleanv' (ulong ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:42'!
alGetBoolean(pname)
	"Alboolean alGetBoolean(ALenum pname);"
	<apicall: bool 'alGetBoolean' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:36'!
alGetBoolean: pname
	"Alboolean alGetBoolean(ALenum pname);"
	<apicall: bool 'alGetBoolean' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:42'!
alGetBufferf(buffer, pname, value)
	"ALvoid alGetBufferf(ALuint buffer,ALenum pname,ALfloat *value)"
	<apicall: void 'alGetBufferf' (ulong ulong float*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/29/2006 14:28'!
alGetBufferf: buffer with: pname

	| value |
	value := FloatArray new: 1.
	self alGetBufferf: buffer with: pname with: value.

	^ value at: 1! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:24'!
alGetBufferf: buffer with: pname with: value
	"ALvoid alGetBufferf(ALuint buffer,ALenum pname,ALfloat *value)"
	<apicall: void 'alGetBufferf' (ulong ulong float*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:42'!
alGetBufferi(buffer, pname, value)
	"ALvoid alGetBufferi(ALuint buffer,ALenum pname,ALint *value)"
	<apicall: void 'alGetBufferi' (ulong ulong long*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/29/2006 15:14'!
alGetBufferi: buffer with: pname

	| holder |
	holder := WordArray new: 1.
	self alGetBufferi: buffer with: pname with: holder.

	^holder at: 1.! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:27'!
alGetBufferi: buffer with: pname with: value
	"ALvoid alGetBufferi(ALuint buffer,ALenum pname,ALint *value)"
	<apicall: void 'alGetBufferi' (ulong ulong long*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:42'!
alGetDoublev(pname, data)
	"ALvoid alGetDoublev(ALenum pname,ALdouble *data);"
	<apicall: void 'alGetDoublev' (ulong double*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:30'!
alGetDoublev: pname with: data
	"ALvoid alGetDoublev(ALenum pname,ALdouble *data);"
	<apicall: void 'alGetDoublev' (ulong double*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:42'!
alGetDouble(pname)
	"Aldouble alGetDouble(ALenum pname);"
	<apicall: double 'alGetDouble' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:35'!
alGetDouble: pname
	"Aldouble alGetDouble(ALenum pname);"
	<apicall: double 'alGetDouble' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/31/2006 15:36'!
alGetError
	"ALenum alGetError(ALvoid);"
	<apicall: ulong 'alGetError' (void) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:43'!
alGetFloatv(pname, data)
	"ALvoid alGetFloatv(ALenum pname,ALfloat *data);"
	<apicall: void 'alGetFloatv' (ulong float*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:31'!
alGetFloatv: pname with: data
	"ALvoid alGetFloatv(ALenum pname,ALfloat *data);"
	<apicall: void 'alGetFloatv' (ulong float*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:43'!
alGetFloat(pname)
	"ALfloat alGetFloat(ALenum pname);"
	<apicall: float 'alGetFloat' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:35'!
alGetFloat: pname
	"ALfloat alGetFloat(ALenum pname);"
	<apicall: float 'alGetFloat' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:43'!
alGetIntegerv(pname, data)
	"ALvoid alGetIntegerv(ALenum pname,ALint *data);"
	<apicall: void 'alGetIntegerv' (ulong long*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:33'!
alGetIntegerv: pname with: data
	"ALvoid alGetIntegerv(ALenum pname,ALint *data);"
	<apicall: void 'alGetIntegerv' (ulong long*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:43'!
alGetInteger(pname)
	"Alint alGetInteger(ALenum pname);"
	<apicall: long 'alGetInteger' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:37'!
alGetInteger: pname
	"Alint alGetInteger(ALenum pname);"
	<apicall: long 'alGetInteger' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/29/2006 15:14'!
alGetListener3fv: pname

	| holder |
	holder := FloatArray new: 3.
	self alGetListenerfv: pname with: holder.

	^ holder! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:44'!
alGetListener3f(pname, v1, v2, v3)
	"ALvoid alGetListener3f(ALenum pname,ALfloat *v1,ALfloat *v2,ALfloat *v3);"
	<apicall: void 'alGetListener3f' (ulong float* float* float*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:38'!
alGetListener3f: pname with: v1 with: v2 with: v3
	"ALvoid alGetListener3f(ALenum pname,ALfloat *v1,ALfloat *v2,ALfloat *v3);"
	<apicall: void 'alGetListener3f' (ulong float* float* float*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/29/2006 15:14'!
alGetListener4fv: pname

	| holder |
	holder := FloatArray new: 4.
	self alGetListenerfv: pname with: holder.

	^ holder! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:44'!
alGetListenerfv(pname, values)
	"ALvoid alGetListenerfv(ALenum pname,ALfloat *values);"
	<apicall: void 'alGetListenerfv' (ulong float*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:39'!
alGetListenerfv: pname with: values
	"ALvoid alGetListenerfv(ALenum pname,ALfloat *values);"
	<apicall: void 'alGetListenerfv' (ulong float*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:45'!
alGetListenerf(pname, value)
	"ALvoid alGetListenerf(ALenum pname,ALfloat *value);"
	<apicall: void 'alGetListenerf' (ulong float*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 8/24/2005 15:39'!
alGetListenerf: pname

	| value |
	value := FloatArray new: 1.
	self alGetListenerf: pname with: value.

	^ value at: 1! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 4/4/2006 15:45'!
alGetListenerf: pname with: value
	"ALvoid alGetListenerf(ALenum pname,ALfloat *value);"
	<apicall: void 'alGetListenerf' (ulong float*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:45'!
alGetListeneri(pname, value)
	"ALvoid alGetListeneri(ALenum pname,ALint *value);"
	<apicall: void 'alGetListeneri' (ulong long*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/29/2006 15:14'!
alGetListeneri: pname

	| holder |
	holder := WordArray new: 1.
	self alGetListeneri: pname with: holder.

	^holder at: 1.! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:11'!
alGetListeneri: pname with: value
	"ALvoid alGetListeneri(ALenum pname,ALint *value);"
	<apicall: void 'alGetListeneri' (ulong long*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/29/2006 15:14'!
alGetSource3fv: source with: pname

	| holder |
	holder := FloatArray new: 3.
	self alGetSourcefv: source with: pname with: holder.

	^ holder! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:46'!
alGetSourcefv(source, pname, value)
	"ALvoid alGetSourcefv(ALuint source,ALenum pname,ALfloat *values);"
	<apicall: void 'alGetSourcefv' (ulong ulong float*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:11'!
alGetSourcefv: source with: pname with: value
	"ALvoid alGetSourcefv(ALuint source,ALenum pname,ALfloat *values);"
	<apicall: void 'alGetSourcefv' (ulong ulong float*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:46'!
alGetSourcef(source, pname, value)
	"ALvoid alGetSourcef(ALuint source,ALenum pname,ALfloat *value);"
	<apicall: void 'alGetSourcef' (ulong ulong float*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/29/2006 13:44'!
alGetSourcef: source with: pname

	| value |
	value := FloatArray new: 1.
	self alGetSourcef: source with: pname with: value.

	^ value at: 1! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:11'!
alGetSourcef: source with: pname with: value
	"ALvoid alGetSourcef(ALuint source,ALenum pname,ALfloat *value);"
	<apicall: void 'alGetSourcef' (ulong ulong float*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:46'!
alGetSourcei(source, pname, value)
	"ALvoid alGetSourcei(ALuint source,ALenum pname,ALint *value);"
	<apicall: void 'alGetSourcei' (ulong ulong long*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/29/2006 15:14'!
alGetSourcei: source with: pname

	| holder |
	holder := WordArray new: 1.
	self alGetSourcei: source with: pname with: holder.

	^holder at: 1.! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:11'!
alGetSourcei: source with: pname with: value
	"ALvoid alGetSourcei(ALuint source,ALenum pname,ALint *value);"
	<apicall: void 'alGetSourcei' (ulong ulong long*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:47'!
alGetString(pname)
	"ALubyte * alGetString(ALenum pname);"
	<apicall: byte* 'alGetString' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:51'!
alGetString: pname
	"ALubyte * alGetString(ALenum pname);"
	<apicall: byte* 'alGetString' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:47'!
alIsBuffer(buffer)
	"Alboolean alIsBuffer(ALuint buffer)"
	<apicall: bool 'alIsBuffer' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:53'!
alIsBuffer: buffer
	"Alboolean alIsBuffer(ALuint buffer)"
	<apicall: bool 'alIsBuffer' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:47'!
alIsEnabled(capability)
	"Alboolean alIsEnabled(ALenum capability);"
	<apicall: bool 'alIsEnabled' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:53'!
alIsEnabled: capability
	"Alboolean alIsEnabled(ALenum capability);"
	<apicall: bool 'alIsEnabled' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:47'!
alIsSource(source)
	"Alboolean alIsSource(ALuint source)"
	<apicall: bool 'alIsSource' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 14:53'!
alIsSource: source
	"Alboolean alIsSource(ALuint source)"
	<apicall: bool 'alIsSource' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:47'!
alListener3f(pname, v1, v2, v3)
	"ALvoid alListener3f(ALenum pname,ALfloat v1,ALfloat v2,ALfloat v3);"
	<apicall: void 'alListener3f' (ulong float float float) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:02'!
alListener3f: pname with: v1 with: v2 with: v3
	"ALvoid alListener3f(ALenum pname,ALfloat v1,ALfloat v2,ALfloat v3);"
	<apicall: void 'alListener3f' (ulong float float float) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:48'!
alListenerfv(pname, values)
	"ALvoid alListenerfv(ALenum pname,ALfloat *values);"
	<apicall: void 'alListenerfv' (ulong float*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:04'!
alListenerfv: pname with: values
	"ALvoid alListenerfv(ALenum pname,ALfloat *values);"
	<apicall: void 'alListenerfv' (ulong float*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:48'!
alListenerf(pname, value)
	"ALvoid alListenerf(ALenum pname,ALfloat value);"
	<apicall: void 'alListenerf' (ulong float) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:04'!
alListenerf: pname with: value
	"ALvoid alListenerf(ALenum pname,ALfloat value);"
	<apicall: void 'alListenerf' (ulong float) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:48'!
alListeneri(pname, value)
	"ALvoid alListeneri(ALenum pname,ALint value);"
	<apicall: void 'alListeneri' (ulong long) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:05'!
alListeneri: pname with: value
	"ALvoid alListeneri(ALenum pname,ALint value);"
	<apicall: void 'alListeneri' (ulong long) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:48'!
alSource3f(source, pname, v1, v2, v3)
	"ALvoid alSource3f(ALuint source,ALenum pname,ALfloat v1,ALfloat v2,ALfloat v3)"
	<apicall: void 'alSource3f' (ulong ulong float float float) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:05'!
alSource3f: source with: pname with: v1 with: v2 with: v3
	"ALvoid alSource3f(ALuint source,ALenum pname,ALfloat v1,ALfloat v2,ALfloat v3)"
	<apicall: void 'alSource3f' (ulong ulong float float float) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:48'!
alSourcefv(source, pname, values)
	"ALvoid alSourcefv(ALuint source,ALenum pname,ALfloat *values)"
	<apicall: void 'alSourcefv' (ulong ulong float*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:06'!
alSourcefv: source with: pname with: values
	"ALvoid alSourcefv(ALuint source,ALenum pname,ALfloat *values)"
	<apicall: void 'alSourcefv' (ulong ulong float*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:49'!
alSourcef(source, pname, value)
	"ALvoid alSourcef(ALuint source,ALenum pname,ALfloat value)"
	<apicall: void 'alSourcef' (ulong ulong float) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:07'!
alSourcef: source with: pname with: value
	"ALvoid alSourcef(ALuint source,ALenum pname,ALfloat value)"
	<apicall: void 'alSourcef' (ulong ulong float) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:49'!
alSourcei(source, pname, value)
	"ALvoid alSourcei(ALuint source,ALenum pname,ALint value)"
	<apicall: void 'alSourcei' (ulong ulong long) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:07'!
alSourcei: source with: pname with: value
	"ALvoid alSourcei(ALuint source,ALenum pname,ALint value)"
	<apicall: void 'alSourcei' (ulong ulong long) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:49'!
alSourcePausev(n, sources)
	"ALvoid alSourcePausev(ALsizei n,ALuint *sources);"
	<apicall: void 'alSourcePausev' (long ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:07'!
alSourcePausev: n with: sources
	"ALvoid alSourcePausev(ALsizei n,ALuint *sources);"
	<apicall: void 'alSourcePausev' (long ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:49'!
alSourcePause(source)
	"ALvoid alSourcePause(ALuint source);"
	<apicall: void 'alSourcePause' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:09'!
alSourcePause: source
	"ALvoid alSourcePause(ALuint source);"
	<apicall: void 'alSourcePause' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:49'!
alSourcePlayv(n, sources)
	"ALvoid alSourcePlayv(ALsizei n,ALuint *sources);"
	<apicall: void 'alSourcePlayv' (long ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:08'!
alSourcePlayv: n with: sources
	"ALvoid alSourcePlayv(ALsizei n,ALuint *sources);"
	<apicall: void 'alSourcePlayv' (long ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:49'!
alSourcePlay(source)
	"ALvoid alSourcePlay(ALint source)"
	<apicall: void 'alSourcePlay' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:09'!
alSourcePlay: source
	"ALvoid alSourcePlay(ALint source)"
	<apicall: void 'alSourcePlay' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:50'!
alSourceQueueBuffers(source, n, buffers)
	"ALvoid alSourceQueueBuffers( ALuint source, ALsizei n, ALuint* buffers );"
	<apicall: void 'alSourceQueueBuffers' (ulong long ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:10'!
alSourceQueueBuffers: source with: n with: buffers
	"ALvoid alSourceQueueBuffers( ALuint source, ALsizei n, ALuint* buffers );"
	<apicall: void 'alSourceQueueBuffers' (ulong long ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:50'!
alSourceRewindv(n, sources)
	"ALvoid alSourceRewindv(ALsizei n,ALuint *sources);"
	<apicall: void 'alSourceRewindv' (long ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:08'!
alSourceRewindv: n with: sources
	"ALvoid alSourceRewindv(ALsizei n,ALuint *sources);"
	<apicall: void 'alSourceRewindv' (long ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:50'!
alSourceRewind(source)
	"ALvoid alSourceRewind(ALuint source);"
	<apicall: void 'alSourceRewind' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:09'!
alSourceRewind: source
	"ALvoid alSourceRewind(ALuint source);"
	<apicall: void 'alSourceRewind' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:50'!
alSourceStopv(n, sources)
	"ALvoid alSourceStopv(ALsizei n,ALuint *sources);"
	<apicall: void 'alSourceStopv' (long ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:08'!
alSourceStopv: n with: sources
	"ALvoid alSourceStopv(ALsizei n,ALuint *sources);"
	<apicall: void 'alSourceStopv' (long ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:50'!
alSourceStop(source)
	"ALvoid alSourceStop(ALuint source);"
	<apicall: void 'alSourceStop' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:09'!
alSourceStop: source
	"ALvoid alSourceStop(ALuint source);"
	<apicall: void 'alSourceStop' (ulong) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'OpenAL API' stamp: 'pbm 4/4/2006 15:50'!
alSourceUnqueueBuffers(source, n, buffers)
	"ALvoid alSourceUnqueueBuffers( ALuint source, ALsizei n, ALuint* buffers );"
	<apicall: void 'alSourceUnqueueBuffers' (ulong long ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/29/2006 15:14'!
alSourceUnqueueBuffers: source with: n

	| holder |
	holder := WordArray new: n.
	self alSourceUnqueueBuffers: source with: n with: holder.

	^ holder! !

!OpenAL methodsFor: 'Keyword API' stamp: 'pbm 3/29/2006 15:10'!
alSourceUnqueueBuffers: source with: n with: buffers
	"ALvoid alSourceUnqueueBuffers( ALuint source, ALsizei n, ALuint* buffers );"
	<apicall: void 'alSourceUnqueueBuffers' (ulong long ulong*) module: 'OpenAL32.dll'>

	^ self externalCallFailed! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/29/2006 15:14'!
alSourceUnqueueBuffer: source

	| holder |
	holder := WordArray new: 1.
	self alSourceUnqueueBuffers: source with: 1 with: holder.

	^ holder at: 1! !

!OpenAL methodsFor: 'utilities' stamp: 'jcg 7/25/2006 00:41'!
checkForError
	| err |
	self flag: #FIXME. "Doesn't work on Windows. Why?"
	Smalltalk platformName = 'Win32' ifTrue: [^self].
	Smalltalk platformName = 'Mac OS' ifTrue: [^self].  "Broken as of OS X 10.4.7"

	(err := self alcGetError) = ALNoError ifFalse: 
		[Transcript show: 'OpenAL Error ', err, ' in ', thisContext sender]! !

!OpenAL methodsFor: 'initialize' stamp: 'jcg 8/19/2006 10:49'!
createContext
	"Open a device and create a new context."
	"Note: we don't make the context current here."
	| device |

	device := self alcOpenDevice: nil.
	device getHandle asInteger > 0 ifFalse: [self error: 'cannot open device'].

	handle := self alcCreateContext: device with: (WordArray with: 0).
	handle getHandle asInteger > 0 ifFalse: [self error: 'cannot create context'].

	^ handle! !

!OpenAL methodsFor: 'finalization' stamp: 'jcg 8/13/2006 22:58'!
destroy

	"Shouldn't be necessary; the names should become invalid once the context is destroyed."
	nameManager deleteAllResources. 
	self destroyContext.
	nameManager destroy.
	nameManager := nil.
! !

!OpenAL methodsFor: 'finalization' stamp: 'jcg 8/6/2006 19:59'!
destroyContext

	| device |

	handle ifNil: [^ self].
	handle isNull ifTrue: [handle := nil. ^ self].

	"Destroy the context and close the device."
	device := self alcGetContextsDevice: handle.
	self isCurrent ifTrue: [
		self alcMakeContextCurrent: (ByteArray with: 0). "Reset the current context."
	].
	self alcDestroyContext: handle.
	self alcCloseDevice: device.  "since we are a singleton, go ahead and destroy the device"

	handle := nil.
	
! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 4/25/2006 11:20'!
device
	^ self alcGetContextsDevice: handle
! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/31/2006 16:16'!
devices
	| device |
	device := self alcGetContextsDevice: handle.
	^ (self alcGetString: device with: AlcDeviceSpecifier) fromCString! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/29/2006 14:58'!
extentions
	^ (self alGetString: ALExtentions) fromCString! !

!OpenAL methodsFor: 'testing' stamp: 'jcg 8/6/2006 17:31'!
hasContext
	^ handle notNil and: [handle isNull not]! !

!OpenAL methodsFor: 'initialize' stamp: 'jcg 7/9/2006 10:27'!
initialize

	self privateInstallLibrary: self openALLibraryName.
	nameManager := OpenALNameManager new initialize: self.
	self createContext.
! !

!OpenAL methodsFor: 'testing' stamp: 'pbm 3/31/2006 11:21'!
isCurrent

	^ (handle getHandle) = ((self alcGetCurrentContext) getHandle)! !

!OpenAL methodsFor: 'testing' stamp: 'pbm 3/23/2006 10:26'!
isPaused: sourceID

	^ (self alGetSourcei: sourceID with: ALSourceState) = ALPaused! !

!OpenAL methodsFor: 'testing' stamp: 'pbm 3/22/2006 16:11'!
isPlaying: sourceID

	^ (self alGetSourcei: sourceID with: ALSourceState) = ALPlaying! !

!OpenAL methodsFor: 'testing' stamp: 'pbm 3/23/2006 10:25'!
isStopped: sourceID

	^ (self alGetSourcei: sourceID with: ALSourceState) = ALStopped! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 4/25/2006 16:49'!
makeCurrent
	self alcMakeContextCurrent: handle.! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 8/18/2005 16:05'!
nameManager
	^ nameManager! !

!OpenAL methodsFor: 'initialize' stamp: 'pbm 3/30/2006 16:38'!
openALLibraryName
	^self subclassResponsibility! !

!OpenAL methodsFor: 'private' stamp: 'pbm 3/30/2006 16:55'!
privateInstallLibrary: openAlLibraryName
	"Do a bit of reflective hacking so that we don't have to carry around lots and lots of duplicate methods for the OpenAL API on each platform. The only 'real' difference is the name of the appropriate shared library here. Indeed, we could (nay... we SHOULD) have done this through a subclass of ExternalLibrary which handles this for us but since we haven't, get out the reflective hammer and nail down the problem."
	| lit |
	InstalledOpenALLibrary = openAlLibraryName ifTrue:[^self].
	OpenAL methodsDo:[:meth|
		lit := meth numLiterals > 0 ifTrue:[meth literalAt: 1].
		lit class == ExternalLibraryFunction ifTrue:[lit setModule: openAlLibraryName].
	].
	InstalledOpenALLibrary := openAlLibraryName.! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/29/2006 14:57'!
renderer
	^ (self alGetString: ALRenderer) fromCString! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/29/2006 14:57'!
vendor
	^ (self alGetString: ALVendor) fromCString! !

!OpenAL methodsFor: 'accessing' stamp: 'pbm 3/30/2006 14:21'!
version
	^ (self alGetString: ALVersion) fromCString! !

!OpenALConstants class methodsFor: 'as yet unclassified' stamp: 'pbm 3/29/2006 17:15'!
initialize

	ALInvalid := -1.
	ALNone := 0.
	ALFalse := 0.
	ALTrue := 1.
	ALNoError := ALFalse.

	ALSourceType := 16r0200.
	ALSourceAbsolute := 16r0201.
	ALSourceRelative := 16r0202.
	ALConeInnerAngle := 16r1001.
	ALConeOuterAngle := 16r1002.
	ALPitch := 16r1003.
	ALPosition := 16r1004.
	ALDirection := 16r1005.
	ALVelocity := 16r1006.
	ALLooping := 16r1007.
	ALStreaming := 16r1008.
	ALBuffer := 16r1009.
	ALGain := 16r100A.
	ALMinGain := 16r100D.
	ALMaxGain := 16r100E.
	ALOrientation := 16r100F.
	ALChannelMask := 16r3000.
	ALSourceState := 16r1010.
	ALInitial := 16r1011.
	ALPlaying := 16r1012.
	ALPaused := 16r1013.
	ALStopped := 16r1014.
	ALBuffersQueued := 16r1015.
	ALBuffersProcessed := 16r1016.
	ALFormatMono8 := 16r1100.
	ALFormatMono16 := 16r1101.
	ALFormatStereo8 := 16r1102.
	ALFormatStereo16 := 16r1103.
	ALReferenceDistance := 16r1020.
	ALRolloffFactor := 16r1021.
	ALConeOuterGain := 16r1022.
	ALMaxDistance := 16r1023.
	ALFrequency := 16r2001.
	ALBits := 16r2002.
	ALChannels := 16r2003.
	ALSize := 16r2004.
	ALData := 16r2005.
	ALUnused := 16r2010.
	ALPending := 16r2011.
	ALProcessed := 16r2012.
	ALDopplerFactor := 16rC000.
	ALDopplerVelocity := 16rC001.
	ALDistanceScale := 16rC002.
	ALDistanceModel := 16rD000.
	ALInverseDistance := 16rD001.
	ALInverseDistanceClamped := 16rD002.

	ALInvalidName := 16rA001.
	ALIllegalEnum := 16rA002.
	ALInvalidEnum := 16rA002.
	ALInvalidValue := 16rA003.
	ALIllegalCommand := 16rA004.
	ALInvalidOperation := 16rA004.
	ALOutOfMemory := 16rA005.

	ALVendor := 16rB001.
	ALVersion := 16rB002.
	ALRenderer := 16rB003.
	ALExtentions := 16rB004.

	ALDopplerFactor := 16rC000.
	ALDopplerVelocity := 16rC001.
	ALSpeedOfSound := 16rC003.

	"check the class pool"
	self classPool keysAndValuesDo:[:key :val|
		val ifNil:[self error: key printString,' undefined'].
	].! !

CroquetHarness subclass: #OpenALHarness
	instanceVariableNames: 'openAL voiceRecorder voiceChats'
	classVariableNames: ''
	poolDictionaries: 'OpenALConstants'
	category: 'OpenAL-Harness'!

!OpenALHarness methodsFor: 'voice chat' stamp: 'pbm 7/13/2006 16:15'!
addVoiceChat: chatID

	| chat |
	chat := TVoiceChat new.
	chat source nameIn: openAL.
	chat play: openAL.
	self voiceChats at: chatID put: chat.

	^ chat! !

!OpenALHarness methodsFor: 'accessing' stamp: 'jcg 8/13/2006 22:58'!
clearSounds
	openAL nameManager deleteAllResources! !

!OpenALHarness methodsFor: 'initialize' stamp: 'jcg 8/13/2006 22:58'!
destroy
	super destroy.

	"Due to context-switching problems, we use a singleton OpenAL instance.  This means that other harnesses may be using the same name manager.  Too bad; they must recreate their handles from their sources."
	openAL nameManager deleteAllResources.
	openAL := nil.

! !

!OpenALHarness methodsFor: 'voice chat' stamp: 'pbm 7/12/2006 13:49'!
endVoiceChat
	(voiceRecorder notNil and: [self isRecording]) ifTrue: [voiceRecorder stopRecording].
	self avatar replica future endVoiceChat: avatar replica.
! !

!OpenALHarness methodsFor: 'initialize' stamp: 'hrs 8/26/2006 16:24'!
initialize
	super initialize.
	openAL := OpenAL new.
	openAL := OpenAL doWithoutErrors: [openAL makeCurrent].
	voiceChats := Dictionary new.! !

!OpenALHarness methodsFor: 'testing' stamp: 'pbm 3/15/2006 16:57'!
isRecording
	^ voiceRecorder notNil and: [ voiceRecorder isRecording ].! !

!OpenALHarness methodsFor: 'accessing' stamp: 'pbm 4/14/2006 15:16'!
listenerGain
	^ 1.0! !

!OpenALHarness methodsFor: 'accessing' stamp: 'pbm 2/17/2006 17:39'!
openAL
	^ openAL! !

!OpenALHarness methodsFor: 'voice chat' stamp: 'pbm 7/13/2006 16:15'!
removeVoiceChat: chatID
	| chat |
	chat := voiceChats removeKey: chatID ifAbsent: [^ nil].
	chat stop: openAL.

	openAL nameManager delete: chat source.

	^ chat! !

!OpenALHarness methodsFor: 'voice chat' stamp: 'liz 5/18/2006 12:58'!
startVoiceChat

	self voiceRecorder streamer: avatar.
	self voiceRecorder startRecording.
	self avatar replica future startVoiceChat: avatar replica.

	! !

!OpenALHarness methodsFor: 'openAL' stamp: 'jcg 8/6/2006 20:57'!
updateListenerPosition: transform

	| position orientation |

	position := transform translation.
	orientation := ((transform lookAt negated) asFloatArray, transform lookUp asFloatArray).

	openAL alListener3f: ALPosition with: position x with: position y with: position z.
	openAL alListenerfv: ALOrientation with: orientation.! !

!OpenALHarness methodsFor: 'accessing' stamp: 'pbm 6/30/2006 13:25'!
voiceChats
	^ voiceChats! !

!OpenALHarness methodsFor: 'accessing' stamp: 'pbm 3/15/2006 16:50'!
voiceRecorder
	^ voiceRecorder ifNil: [
		voiceRecorder := TVoiceRecorder new samplingRate: 11025; recordLevel: 1 ].! !

Object subclass: #OpenALName
	instanceVariableNames: 'manager alID'
	classVariableNames: ''
	poolDictionaries: 'OpenALConstants'
	category: 'OpenAL-Base'!

!OpenALName commentStamp: '<historical>' prior: 0!
Base class for OpenAL resources that need a unique integer identifier. OpenALNameManager maps the TOpenALResource objects to their respective OpenALName (i.e. OpenALBufferName and OpenALSourceName).!

OpenALName subclass: #OpenALBufferName
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenAL-Base'!

!OpenALBufferName commentStamp: '<historical>' prior: 0!
I communicate directly with the OpenAL object to update properties of my associated TSoundBuffer object.!

!OpenALBufferName class methodsFor: 'accessing' stamp: 'pbm 8/18/2005 15:47'!
dictionarySelector
	"Method selector used by OpenALNameManager to determine in which dictionary names of this class are stored."

	^ #buffers! !

!OpenALBufferName methodsFor: 'private' stamp: 'pbm 3/22/2006 16:04'!
generateName: openAL
	^ openAL alGenBuffer! !

!OpenALBufferName methodsFor: 'initialize' stamp: 'jcg 7/24/2006 23:59'!
initialize: openAL forResource: buffer "a TSoundBuffer"
	super initialize: openAL forResource: buffer.
	self updateFromBuffer: buffer! !

!OpenALBufferName methodsFor: 'private' stamp: 'pbm 2/15/2006 14:47'!
privateDeleteIn: openAL

	openAL alDeleteBuffers: 1 with: (WordArray with: alID).! !

!OpenALBufferName methodsFor: 'private' stamp: 'jcg 7/24/2006 23:57'!
updateFromBuffer: buffer "aTSoundBuffer"
	buffer size > 0 ifTrue: [
		self openAL 
				alBufferData: alID
				with: buffer format
				with: buffer data
				with: buffer size
				with: buffer samplingRate.
	].! !

!OpenALName class methodsFor: 'as yet unclassified' stamp: 'pbm 8/23/2005 11:48'!
dictionarySelector
	"Method selector used by OpenALNameManager to determine in which dictionary names of this class are stored."
	self subclassResponsibility! !

!OpenALName class methodsFor: 'as yet unclassified' stamp: 'pbm 8/23/2005 11:47'!
new
	
	self shouldNotImplement.	"use #new:forResource: instead"! !

!OpenALName class methodsFor: 'as yet unclassified' stamp: 'pbm 2/15/2006 15:39'!
new: openAL forResource: anOpenALResource

	^ super new initialize: openAL forResource: anOpenALResource! !

!OpenALName methodsFor: 'accessing' stamp: 'pbm 8/18/2005 14:31'!
alID
	"Answer the low-level OpenAL name (ie: an integer) for this resource."
	^alID! !

!OpenALName methodsFor: 'initialize' stamp: 'pbm 3/9/2006 13:57'!
deleteIn: openAL
	"Subclasses know how to free the type of OpenAL resource that they reference."
	
	self privateDeleteIn: openAL.
	alID := nil.
	manager := nil.! !

!OpenALName methodsFor: 'private' stamp: 'pbm 2/15/2006 15:35'!
generateName: openAL
	"Call the appropriate OpenAL code to create a new 'name', appropriate to the type of resource.  This simply reserves a name; the resource is not yet ready to use.  See subclasses for examples."

	self subclassResponsibility! !

!OpenALName methodsFor: 'initialize' stamp: 'jcg 7/25/2006 00:17'!
initialize: openAL forResource: anOpenALResource
	"This method simply generates a resource name.  Subclasses should additionally do whatever is necessary to initialize the resource that the name refers to.  See subclasses for concrete examples."
	manager := openAL nameManager.
	alID := self generateName: openAL.
! !

!OpenALName methodsFor: 'invalidation' stamp: 'pbm 8/24/2005 17:11'!
invalidate: anOpenALResource
	"By default, just delete ourselves from our manager and allow ourself to be lazily re-created.  Subclasses might do something fancier for efficiency."

	manager delete: anOpenALResource! !

!OpenALName methodsFor: 'accessing' stamp: 'jcg 7/25/2006 00:06'!
openAL
	^ manager openAL! !

!OpenALName methodsFor: 'message handling' stamp: 'setup 9/18/2005 10:50'!
perform: selector withArguments: args forNotifier: notifier
	
	alID ifNotNil: [^ self perform: selector withArguments: args].

	"Only happens if we have already been deleted; don't want further notifications.  I'm not sure if this is even necessary, since #delete sends #terminateScript to all scripts (doing so does remove this object from the notifier's list of objects to broadcast to, right?)."
	self stopScript: #perform:withArguments:forNotifier:.
	Transcript show: 'removed notification'; cr.! !

!OpenALName methodsFor: 'private' stamp: 'pbm 2/15/2006 14:43'!
privateDeleteIn: openAL
	"Subclasses know how to free the type of OpenAL resource that they reference."

	self subclassResponsibility.! !

!OpenALName methodsFor: 'private' stamp: 'pbm 1/6/2006 16:45'!
set: varName to: newValue
	newValue ifNotNil: [
		^ self perform: (varName,':') asSymbol with: newValue
	]! !

OpenALName subclass: #OpenALSourceName
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenAL-Base'!

!OpenALSourceName commentStamp: '<historical>' prior: 0!
I communicate directly with the OpenAL object to update properties of my associated TSoundSource object.!

!OpenALSourceName class methodsFor: 'accessing' stamp: 'pbm 8/18/2005 15:47'!
dictionarySelector
	"Method selector used by OpenALNameManager to determine in which dictionary names of this class are stored."

	^ #sources! !

!OpenALSourceName methodsFor: 'accessing' stamp: 'pbm 3/22/2006 15:14'!
buffersProcessed

	^ self openAL alGetSourcei: alID with: ALBuffersProcessed! !

!OpenALSourceName methodsFor: 'accessing' stamp: 'jcg 7/25/2006 00:52'!
buffer: bufferName
	self openAL alSourcei: alID with: ALBuffer with: bufferName alID! !

!OpenALSourceName methodsFor: 'accessing' stamp: 'pbm 9/19/2005 15:09'!
coneInnerAngle: value

	self openAL alSourcef: self alID with: ALConeInnerAngle with: value! !

!OpenALSourceName methodsFor: 'accessing' stamp: 'pbm 9/19/2005 15:09'!
coneOuterAngle: value

	self openAL alSourcef: self alID with: ALConeOuterAngle with: value! !

!OpenALSourceName methodsFor: 'accessing' stamp: 'pbm 9/19/2005 15:13'!
coneOuterGain: value

	self openAL alSourcef: self alID with: ALConeOuterGain with: value! !

!OpenALSourceName methodsFor: 'accessing' stamp: 'pbm 3/31/2006 14:06'!
direction: value

	self openAL alSourcefv: self alID with: ALDirection with: (value asFloatArray).! !

!OpenALSourceName methodsFor: 'accessing' stamp: 'pbm 9/19/2005 14:24'!
gain: value

	self openAL alSourcef: self alID with: ALGain with: value! !

!OpenALSourceName methodsFor: 'private' stamp: 'pbm 3/22/2006 16:04'!
generateName: openAL
	^ openAL alGenSource! !

!OpenALSourceName methodsFor: 'initialize' stamp: 'jcg 7/25/2006 00:52'!
initialize: openAL forResource: source "a TSoundSource"

	super initialize: openAL forResource: source.

	"Force the sound source to update it's state."
	source properties keysAndValuesDo: [:key :value|
		self set: key to: value.
	].
	source buffer ifNotNilDo: [:buf | self buffer: (buf nameIn: openAL)].
	source isPlaying ifTrue: [self play].
	source isPaused ifTrue: [self pause].! !

!OpenALSourceName methodsFor: 'testing' stamp: 'pbm 9/22/2005 12:25'!
isPaused

	^ self state = ALPaused! !

!OpenALSourceName methodsFor: 'testing' stamp: 'pbm 9/21/2005 16:00'!
isPlaying

	^ self state = ALPlaying! !

!OpenALSourceName methodsFor: 'testing' stamp: 'pbm 9/22/2005 12:25'!
isStopped

	^ self state = ALStopped! !

!OpenALSourceName methodsFor: 'accessing' stamp: 'jcg 7/25/2006 00:08'!
looping: value
	self openAL alSourcei: alID with: ALLooping with: (value ifTrue: [1] ifFalse: [0]).! !

!OpenALSourceName methodsFor: 'accessing' stamp: 'pbm 9/19/2005 15:16'!
maxDistance: value

	self openAL alSourcef: self alID with: ALMaxDistance with: value! !

!OpenALSourceName methodsFor: 'controls' stamp: 'pbm 8/26/2005 13:10'!
pause

	self openAL alSourcePause: alID! !

!OpenALSourceName methodsFor: 'accessing' stamp: 'pbm 9/19/2005 15:16'!
pitch: value

	self openAL alSourcef: self alID with: ALPitch with: value! !

!OpenALSourceName methodsFor: 'controls' stamp: 'pbm 8/26/2005 13:10'!
play

	self openAL alSourcePlay: alID! !

!OpenALSourceName methodsFor: 'accessing' stamp: 'pbm 3/31/2006 14:06'!
position: value

	self openAL alSourcefv: self alID with: ALPosition with: (value asFloatArray).! !

!OpenALSourceName methodsFor: 'private' stamp: 'jcg 7/25/2006 01:16'!
privateDeleteIn: openAL
	"You would expect that deleting a source would stop it from playing, but (at least on the Mac) this is not true."
	openAL 
		alSourceStop: alID;
		alDeleteSources: 1 with: (WordArray with: alID).
! !

!OpenALSourceName methodsFor: 'accessing' stamp: 'pbm 9/19/2005 15:18'!
referenceDistance: value

	self openAL alSourcef: self alID with: ALReferenceDistance with: value! !

!OpenALSourceName methodsFor: 'accessing' stamp: 'pbm 1/4/2006 17:20'!
relative

	^ self openAL alGetSourcei: alID with: ALSourceRelative! !

!OpenALSourceName methodsFor: 'controls' stamp: 'pbm 8/26/2005 13:10'!
rewind

	self openAL alSourceRewind: alID! !

!OpenALSourceName methodsFor: 'accessing' stamp: 'pbm 9/19/2005 15:18'!
rolloffFactor: value

	self openAL alSourcef: self alID with: ALRolloffFactor with: value! !

!OpenALSourceName methodsFor: 'accessing' stamp: 'jcg 7/26/2006 00:38'!
samplingRate: rate
	"No-op.  Deprecated."
	self deprecatedExplanation: 'Source does not have an intrinsic sample rate; sound buffers do'! !

!OpenALSourceName methodsFor: 'accessing' stamp: 'pbm 9/20/2005 16:26'!
state

	^ self openAL alGetSourcei: alID with: ALSourceState! !

!OpenALSourceName methodsFor: 'controls' stamp: 'pbm 8/26/2005 13:10'!
stop

	self openAL alSourceStop: alID! !

Object subclass: #OpenALNameManager
	instanceVariableNames: 'openAL buffers sources'
	classVariableNames: ''
	poolDictionaries: 'OpenALConstants'
	category: 'OpenAL-Base'!

!OpenALNameManager commentStamp: '<historical>' prior: 0!
I map TOpenALResource objects to a name in OpenAL (a unique integer).!

!OpenALNameManager methodsFor: 'private' stamp: 'pbm 8/18/2005 16:57'!
buffers
	^ buffers! !

!OpenALNameManager methodsFor: 'accessing' stamp: 'jcg 8/13/2006 22:58'!
deleteAllResources

	{sources. buffers} do: [:dict |
		dict values do: [:val | val deleteIn: openAL]
	].
	buffers := Dictionary new.
	sources := Dictionary new.

! !

!OpenALNameManager methodsFor: 'accessing' stamp: 'jcg 7/6/2006 18:03'!
delete: anOpenALResource
	"If name exists for resource, remove and delete it."
	| name |
	name := (self dictFor: anOpenALResource) removeKey: anOpenALResource objectID ifAbsent: [^ self].
	name deleteIn: openAL.
! !

!OpenALNameManager methodsFor: 'initialize' stamp: 'jcg 8/6/2006 20:01'!
destroy
	"Called when the OpenAL is destroyed, so it is not necessary for the manager to destroy OpenALNames.  This might be a bit confusing?"
	openAL := nil.! !

!OpenALNameManager methodsFor: 'private' stamp: 'pbm 8/18/2005 15:06'!
dictFor: aResource
	"Each type of OpenALResource is stored in a separate dictionary; answer the appropriate dictionary for this anOpenALResource."
	^ self perform: aResource resourceNameClass dictionarySelector! !

!OpenALNameManager methodsFor: 'initialize' stamp: 'jcg 7/6/2006 18:15'!
initialize: anOpenAL

	openAL := anOpenAL.

	buffers := Dictionary new.
	sources := Dictionary new.! !

!OpenALNameManager methodsFor: 'accessing' stamp: 'jcg 7/4/2006 21:18'!
nameOf: anOpenALResource
	"Answers an OpenAL name for the given resource.  Instantiate one if necessary."
	| nameClass dict name |
	nameClass := anOpenALResource resourceNameClass.
	dict := self perform: nameClass dictionarySelector.
	^ dict 
		at: anOpenALResource objectID
		ifAbsentPut: 
			[name := nameClass new: openAL forResource: anOpenALResource.
			name]! !

!OpenALNameManager methodsFor: 'accessing' stamp: 'jcg 7/6/2006 18:10'!
nameOf: aResource ifAbsent: aBlock
	"Answers an OpenAL name for the given resource."

	^ (self dictFor: aResource) at: aResource objectID ifAbsent: aBlock! !

!OpenALNameManager methodsFor: 'accessing' stamp: 'pbm 8/22/2005 16:56'!
openAL
	^ openAL! !

!OpenALNameManager methodsFor: 'private' stamp: 'pbm 8/18/2005 16:57'!
sources
	^ sources! !

TestCase subclass: #OpenALTest
	instanceVariableNames: 'openAL'
	classVariableNames: ''
	poolDictionaries: 'OpenALConstants AlcConstants'
	category: 'OpenAL-Tests'!

!OpenALTest commentStamp: 'mpm 4/7/2006 17:01' prior: 0!
A simple test to make sure OpenAL is functioning properly
If it works right you'll be rewarded with a "clink" sound.

run the test like this:

| foo |
  foo := OpenALTest new.
  foo setUp.
  foo testOpenAL.
  foo tearDown.
!

!OpenALTest methodsFor: 'running' stamp: 'pbm 4/5/2006 15:04'!
setUp
	openAL := OpenAL new.! !

!OpenALTest methodsFor: 'running' stamp: 'pbm 4/5/2006 15:06'!
tearDown

	openAL destroy.
! !

!OpenALTest methodsFor: 'testing' stamp: 'pbm 4/7/2006 15:35'!
testOpenAL

	| device context contextsDevice current source buffer gain data buf |

	"Clear the error codes."
"	openAL alcGetError."
	openAL alGetError.

	"Test openening a device."
	device := openAL alcOpenDevice: (ByteArray with: 0).
"	self assert: (openAL alcGetError = AlcNoError)."
	self assert: (device isKindOf: ExternalData).
	self assert: (device getHandle asInteger) > 0.

	"Test creating a context."
	context := openAL alcCreateContext: device with: (WordArray with: 0).
"	self assert: (openAL alcGetError = AlcNoError)."
	self assert: (context isKindOf: ExternalData).
	self assert: (context getHandle asInteger) > 0.

	"Test making the context current."
	openAL alcMakeContextCurrent: context.
"	self assert: (openAL alcGetError = AlcNoError)."
	current := openAL alcGetCurrentContext.
"	self assert: (openAL alcGetError = AlcNoError)."
	self assert: (current getHandle) = (context getHandle).

	"Test generating a source."
	source := openAL alGenSource.
	self assert: (openAL alGetError = ALNoError).
	self assert: (source isInteger).
	self assert: (source > 0).
	self assert: (openAL alIsSource: source).

	"Test setting/getting source gain."
	openAL alSourcef: source with: ALGain with: 0.75.
	self assert: (openAL alGetError = ALNoError).
	gain := openAL alGetSourcef: source with: ALGain.
	self assert: (openAL alGetError = ALNoError).
	self assert: (gain = 0.75).

	"Test generating a buffer."
	buffer := openAL alGenBuffer.
	self assert: (openAL alGetError = ALNoError).
	self assert: (buffer isInteger).
	self assert: (buffer > 0).
	self assert: (openAL alIsBuffer: buffer).

	"Test filling a buffer with samples."
	data := SampledSound coffeeCupClink.
	openAL alBufferData: buffer with: ALFormatMono16 with: data with: data size with: 12000.
	self assert: (openAL alGetError = ALNoError).
	self assert: (openAL alGetBufferi: buffer with: ALSize) = data size.
	
	"Test assigning a buffer to a source."
	openAL alSourcei: source with: ALBuffer with: buffer.
	self assert: (openAL alGetError = ALNoError).
	buf := openAL alGetSourcei: source with: ALBuffer.
	self assert: (openAL alGetError = ALNoError).
	self assert: (buf = buffer).

	"Test playing the source."
	openAL alSourcePlay: source.
	self assert: (openAL alGetError = ALNoError).

	self wait: 0.5.

	"Test deleting the source."
	openAL alDeleteSource: source.
	self assert: (openAL alGetError = ALNoError).

	"Test deleting the buffer."
	openAL alDeleteBuffer: buffer.
	self assert: (openAL alGetError = ALNoError).

	"Test getting the context's device."
	contextsDevice := openAL alcGetContextsDevice: context.
"	self assert: (openAL alcGetError = AlcNoError)."
	self assert: (contextsDevice getHandle) = (device getHandle).

	"Test resetting the current context."
	openAL alcMakeContextCurrent: nil.
"	self assert: (openAL alcGetError = AlcNoError)."
	current := openAL alcGetCurrentContext.
"	self assert: (openAL alcGetError = AlcNoError)."
	self assert: (current isKindOf: ExternalData).
	self assert: (current getHandle asInteger) = 0.

	"Test destroying the context."
	openAL alcDestroyContext: context.
"	self assert: (openAL alcGetError = AlcNoError)."

	"Test closing the device."
	openAL alcCloseDevice: device.
"	self assert: (openAL alcGetError = AlcNoError).	"! !

Object subclass: #TOpenALResource
	instanceVariableNames: 'properties objectID'
	classVariableNames: ''
	poolDictionaries: 'OpenALConstants'
	category: 'OpenAL-Objects'!

!TOpenALResource commentStamp: '<historical>' prior: 0!
Base class for OpenAL resources that maintain state in the replicated space. I must be mapped to an instance of an OpenALName via #nameIn:. I communicate to my OpenALName by sending messages to a Notifier object who translates those messages into events that are registered with my OpenALName instance.

*** Is it ok to use triggerEvent:withArguments: and when:send:to: for this communication? I tried to use the more familiar scripting stuff but it wouldn't work across islands.!

!TOpenALResource methodsFor: 'initialize' stamp: 'jcg 7/25/2006 00:15'!
delete
	self invalidate.! !

!TOpenALResource methodsFor: 'accessing' stamp: 'jcg 5/12/2006 00:59'!
handle
	"Answer a low-level resource name for this resource in the given OpenAL context."
	^ self openAL ifNotNilDo: [:oal | oal nameManager nameOf: self]! !

!TOpenALResource methodsFor: 'initialize' stamp: 'jcg 7/4/2006 21:17'!
initialize
	super initialize.
	properties := Dictionary new.
	objectID := TObjectID new.! !

!TOpenALResource methodsFor: 'invalidation' stamp: 'jcg 5/12/2006 00:50'!
invalidate
	self handle invalidate! !

!TOpenALResource methodsFor: 'accessing' stamp: 'pbm 8/24/2005 16:58'!
nameIn: openAL
	"Answer a low-level resource name for this resource in the given OpenAL context."
	^ openAL nameManager nameOf: self! !

!TOpenALResource methodsFor: 'accessing' stamp: 'jcg 7/4/2006 21:18'!
objectID
	^ objectID copy! !

!TOpenALResource methodsFor: 'private' stamp: 'jcg 7/25/2006 00:12'!
openAL
	^ OpenAL singleton! !

!TOpenALResource methodsFor: 'accessing' stamp: 'pbm 8/18/2005 15:13'!
resourceNameClass
	"Class of the corresponding OpenALName subclass."

	self subclassResponsibility.! !

TOpenALResource subclass: #TSoundBuffer
	instanceVariableNames: 'data format size samplingRate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenAL-Objects'!

!TSoundBuffer commentStamp: '<historical>' prior: 0!
I represent a buffer of sound for attaching to a TSoundSource.!

!TSoundBuffer class methodsFor: 'as yet unclassified' stamp: 'pbm 1/9/2006 12:55'!
data: soundBuffer samplingRate: frequency format: formatToken
	^ self new data: soundBuffer samplingRate: frequency format: formatToken! !

!TSoundBuffer class methodsFor: 'as yet unclassified' stamp: 'pbm 3/2/2006 11:42'!
data: soundBuffer samplingRate: frequency sampleSize: sampleSize stereo: stereoFlag
	^ self new data: soundBuffer 
		samplingRate: frequency 
		sampleSize: sampleSize 
		stereo: stereoFlag! !

!TSoundBuffer methodsFor: 'accessing' stamp: 'pbm 2/27/2006 15:46'!
data
	^ data! !

!TSoundBuffer methodsFor: 'accessing' stamp: 'jcg 5/12/2006 01:00'!
data: soundBuffer samplingRate: frequency format: formatToken
	data := soundBuffer.
	format := formatToken.
	size := soundBuffer byteSize.
	samplingRate := frequency asInteger.

	self handle ifNotNilDo: [:h | h updateFromBuffer: self]
! !

!TSoundBuffer methodsFor: 'accessing' stamp: 'pbm 9/1/2005 13:22'!
data: soundBuffer samplingRate: frequency sampleSize: sampleSize stereo: stereoFlag

	| formatToken |

	stereoFlag ifTrue: [
		sampleSize = 16 ifTrue: [ formatToken := ALFormatStereo16 ].
		sampleSize = 8 ifTrue: [ formatToken := ALFormatStereo8 ].
	] ifFalse: [
		sampleSize = 16 ifTrue: [ formatToken := ALFormatMono16 ].
		sampleSize = 8 ifTrue: [ formatToken := ALFormatMono8 ].
	].

	formatToken ifNil: [ ^ self ].

	self data: soundBuffer samplingRate: frequency format: formatToken
! !

!TSoundBuffer methodsFor: 'accessing' stamp: 'pbm 3/1/2006 17:00'!
format
	^ format! !

!TSoundBuffer methodsFor: 'initialize' stamp: 'pbm 3/3/2006 13:59'!
initialize
	super initialize.
	size = 0.! !

!TSoundBuffer methodsFor: 'testing' stamp: 'pbm 3/3/2006 13:59'!
isEmpty
	^ size = 0! !

!TSoundBuffer methodsFor: 'accessing' stamp: 'pbm 8/18/2005 15:16'!
resourceNameClass
	"Class of the corresponding OpenALName subclass."
	^ OpenALBufferName! !

!TSoundBuffer methodsFor: 'accessing' stamp: 'pbm 3/1/2006 17:00'!
samplingRate
	^ samplingRate! !

!TSoundBuffer methodsFor: 'accessing' stamp: 'pbm 3/1/2006 17:01'!
size
	^ size! !

TOpenALResource subclass: #TSoundSource
	instanceVariableNames: 'buffer processed queued state streaming'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OpenAL-Objects'!

!TSoundSource commentStamp: '<historical>' prior: 0!
I maintain the state of a sound source and provide an interface for updating my properties.!

!TSoundSource methodsFor: 'accessing' stamp: 'pbm 1/6/2006 16:04'!
buffer
	^ buffer! !

!TSoundSource methodsFor: 'accessing' stamp: 'pbm 1/6/2006 13:35'!
buffersProcessed
	^ processed! !

!TSoundSource methodsFor: 'accessing' stamp: 'pbm 1/6/2006 13:35'!
buffersQueued
	^ queued! !

!TSoundSource methodsFor: 'buffering' stamp: 'jcg 7/9/2006 11:53'!
buffer: aTSoundBuffer
	buffer := aTSoundBuffer.
	self handle ifNotNilDo: [:h | h buffer: aTSoundBuffer handle]! !

!TSoundSource methodsFor: 'accessing' stamp: 'pbm 9/19/2005 16:14'!
coneInnerAngle

	^ properties at: #coneInnerAngle! !

!TSoundSource methodsFor: 'accessing' stamp: 'jcg 5/12/2006 10:01'!
coneInnerAngle: aFloat

	aFloat isNumber ifFalse: [ ^self ].
	properties at: #coneInnerAngle put: aFloat asFloat.
	self handle ifNotNilDo: [:h | h coneInnerAngle: aFloat asFloat].! !

!TSoundSource methodsFor: 'accessing' stamp: 'pbm 9/19/2005 16:14'!
coneOuterAngle

	^ properties at: #coneOuterAngle! !

!TSoundSource methodsFor: 'accessing' stamp: 'jcg 5/12/2006 10:02'!
coneOuterAngle: aFloat

	aFloat isNumber ifFalse: [ ^self ].
	properties at: #coneOuterAngle put: aFloat asFloat.
	self handle ifNotNilDo: [:h | h coneOuterAngle: aFloat asFloat].! !

!TSoundSource methodsFor: 'accessing' stamp: 'pbm 9/19/2005 16:14'!
coneOuterGain

	^ properties at: #coneOuterGain! !

!TSoundSource methodsFor: 'accessing' stamp: 'jcg 5/12/2006 10:02'!
coneOuterGain: aFloat

	aFloat isNumber ifFalse: [ ^self ].
	properties at: #coneOuterGain put: aFloat asFloat.
	self handle ifNotNilDo: [:h | h coneOuterGain: aFloat asFloat].! !

!TSoundSource methodsFor: 'initialize' stamp: 'pbm 1/9/2006 13:21'!
delete
	super delete.
	buffer ifNotNil: [
		buffer delete.
		buffer := nil.
	].! !

!TSoundSource methodsFor: 'accessing' stamp: 'pbm 2/20/2006 16:46'!
direction

	| direction |
	direction := properties at: #direction.
	^ Vector3 x: (direction at: 1) y: (direction at: 2) z: (direction at: 3).! !

!TSoundSource methodsFor: 'accessing' stamp: 'jcg 5/12/2006 10:03'!
direction: aVector
	properties at: #direction put: aVector.
	self handle ifNotNilDo: [:h | h direction: aVector].
! !

!TSoundSource methodsFor: 'accessing' stamp: 'pbm 9/19/2005 16:15'!
gain

	^ properties at: #gain! !

!TSoundSource methodsFor: 'accessing' stamp: 'jcg 5/12/2006 10:03'!
gain: aFloat
	aFloat isNumber ifFalse: [ ^self ].
	properties at: #gain put: aFloat asFloat.
	self handle ifNotNilDo: [:h | h gain: aFloat asFloat].! !

!TSoundSource methodsFor: 'initialize' stamp: 'pbm 1/6/2006 16:25'!
initialize

	super initialize.
	buffer := nil.
	state := ALInitial.
	queued := processed := 0.
	streaming := false.

	"Set the initial values for all properties."
	properties at: #coneInnerAngle put: 360.0.
	properties at: #coneOuterAngle put: 360.0.
	properties at: #coneOuterGain put: 0.0.
	properties at: #direction put: #(0.0 0.0 0.0).
	properties at: #gain put: 0.5.
	properties at: #looping put: false.
	properties at: #maxDistance put: 1000.0.
	properties at: #pitch put: 1.0.
	properties at: #position put: #(0.0 0.0 0.0).
	properties at: #referenceDistance put: 1.0.
	properties at: #rolloffFactor put: 1.0.! !

!TSoundSource methodsFor: 'testing' stamp: 'pbm 8/21/2005 15:44'!
isPaused
	^ self state = ALPaused! !

!TSoundSource methodsFor: 'testing' stamp: 'pbm 8/18/2005 18:06'!
isPlaying
	^ self state = ALPlaying! !

!TSoundSource methodsFor: 'testing' stamp: 'pbm 8/21/2005 15:44'!
isStopped
	^ self state = ALStopped! !

!TSoundSource methodsFor: 'testing' stamp: 'pbm 1/5/2006 15:59'!
isStreaming
	^ streaming! !

!TSoundSource methodsFor: 'accessing' stamp: 'pbm 9/19/2005 16:15'!
looping

	^ properties at: #looping! !

!TSoundSource methodsFor: 'accessing' stamp: 'jcg 5/12/2006 10:03'!
looping: aBool
	properties at: #looping put: aBool.
	self handle ifNotNilDo: [:h | h looping: aBool].! !

!TSoundSource methodsFor: 'accessing' stamp: 'pbm 9/19/2005 16:15'!
maxDistance

	^ properties at: #maxDistance! !

!TSoundSource methodsFor: 'accessing' stamp: 'jcg 5/12/2006 10:04'!
maxDistance: aFloat
	aFloat isNumber ifFalse: [ ^self ].
	properties at: #maxDistance put: aFloat asFloat.
	self handle ifNotNilDo: [:h | h maxDistance: aFloat asFloat].! !

!TSoundSource methodsFor: 'controls' stamp: 'jcg 5/12/2006 01:08'!
pause
	self handle ifNotNilDo: [:h | h pause].
	state := ALPaused.! !

!TSoundSource methodsFor: 'accessing' stamp: 'pbm 9/19/2005 16:15'!
pitch

	^ properties at: #pitch! !

!TSoundSource methodsFor: 'accessing' stamp: 'jcg 5/12/2006 10:04'!
pitch: aFloat
	aFloat isNumber ifFalse: [ ^self ].
	properties at: #pitch put: aFloat asFloat.
	self handle ifNotNilDo: [:h | h pitch: aFloat asFloat].! !

!TSoundSource methodsFor: 'controls' stamp: 'jcg 5/12/2006 01:08'!
play
	self handle ifNotNilDo: [:h | h play].
	state := ALPlaying.! !

!TSoundSource methodsFor: 'accessing' stamp: 'pbm 2/20/2006 16:46'!
position

	| position |
	position := properties at: #position.
	^ Vector3 x: (position at: 1) y: (position at: 2) z: (position at: 3)! !

!TSoundSource methodsFor: 'accessing' stamp: 'jcg 5/12/2006 10:05'!
position: aVector3
	properties at: #position put: aVector3.
	self handle ifNotNilDo: [:h | h position: aVector3].! !

!TSoundSource methodsFor: 'accessing' stamp: 'pbm 6/30/2006 15:57'!
position: posVector direction: dirVector

	self position: posVector; direction: dirVector.! !

!TSoundSource methodsFor: 'accessing' stamp: 'pbm 1/6/2006 13:08'!
properties
	^ properties! !

!TSoundSource methodsFor: 'accessing' stamp: 'pbm 9/19/2005 16:15'!
referenceDistance

	^ properties at: #referenceDistance! !

!TSoundSource methodsFor: 'accessing' stamp: 'jcg 5/12/2006 10:05'!
referenceDistance: aFloat
	aFloat isNumber ifFalse: [ ^self ].
	properties at: #referenceDistance put: aFloat asFloat.
	self handle ifNotNilDo: [:h | h referenceDistance: aFloat asFloat].! !

!TSoundSource methodsFor: 'render' stamp: 'pbm 4/19/2006 10:58'!
render: openAL

	| sourceID |

	openAL isCurrent ifFalse: [ ^ 0 ].

	sourceID := (self nameIn: openAL) alID.

	streaming ifTrue: [
		processed := openAL alGetSourcei: sourceID with: ALBuffersProcessed.
		queued := openAL alGetSourcei: sourceID with: ALBuffersQueued.
	].

	state := openAL alGetSourcei: sourceID with: ALSourceState.! !

!TSoundSource methodsFor: 'accessing' stamp: 'pbm 8/18/2005 15:16'!
resourceNameClass
	"Class of the corresponding OpenALName subclass."
	^ OpenALSourceName! !

!TSoundSource methodsFor: 'controls' stamp: 'jcg 5/12/2006 01:08'!
rewind
	self handle ifNotNilDo: [:h | h rewind].
	state := ALStopped.! !

!TSoundSource methodsFor: 'accessing' stamp: 'pbm 9/19/2005 16:15'!
rolloffFactor

	^ properties at: #rolloffFactor! !

!TSoundSource methodsFor: 'accessing' stamp: 'jcg 5/12/2006 10:06'!
rolloffFactor: aFloat
	aFloat isNumber ifFalse: [ ^self ].
	properties at: #rolloffFactor put: aFloat asFloat.
	self handle ifNotNilDo: [:h | h rolloffFactor: aFloat asFloat].! !

!TSoundSource methodsFor: 'accessing' stamp: 'pbm 1/6/2006 13:33'!
state
	^ state! !

!TSoundSource methodsFor: 'controls' stamp: 'jcg 5/12/2006 01:08'!
stop
	self handle ifNotNilDo: [:h | h stop].
	state := ALStopped.! !

!TSoundSource methodsFor: 'accessing' stamp: 'pbm 1/5/2006 15:59'!
streaming: aBool
	streaming := aBool.! !

!TSoundSource methodsFor: 'accessing' stamp: 'pbm 7/13/2006 16:18'!
transform: trans
	self position: trans translation.
	self direction: trans column3 normalized.! !

!TSoundSource methodsFor: 'accessing' stamp: 'jcg 5/12/2006 10:06'!
x: x y: y z: z
	self position: x@y@z! !

TFrame subclass: #TSoundFrame
	instanceVariableNames: 'source'
	classVariableNames: ''
	poolDictionaries: 'OpenALConstants'
	category: 'OpenAL-Objects'!

!TSoundFrame commentStamp: '<historical>' prior: 0!
TSoundFrame is a convienient way of getting a sound into a shared space. It works by overriding #frameChanged to update the position and direction of it's TSoundSource object. It also piggybacks on the rendering loop (which gives access to the OpenAL object via the harness) in order to get a "name" for the sound source as soon as it's added to a space.!

!TSoundFrame methodsFor: 'initialize' stamp: 'pbm 2/17/2006 17:19'!
destroy
	source delete.
	super destroy.
! !

!TSoundFrame methodsFor: 'accessing' stamp: 'pbm 2/23/2006 14:01'!
frameChanged

	super frameChanged.
	source position: self globalPosition.
	source direction: self lookAt.! !

!TSoundFrame methodsFor: 'accessing' stamp: 'pbm 3/23/2006 13:29'!
gain: aNumber
	source gain: aNumber.! !

!TSoundFrame methodsFor: 'initialize' stamp: 'pbm 2/17/2006 17:19'!
initialize

	super initialize.
	source := TSoundSource new.! !

!TSoundFrame methodsFor: 'loading' stamp: 'etw 4/17/2006 15:47'!
loadData: data

	|  format buf samplingRate rawBuf |
	rawBuf := SoundBuffer fromByteArray: data.
	samplingRate :=  44100.
	format := ALFormatMono16.

	buf := TSoundBuffer data: rawBuf samplingRate: samplingRate format: format.
	source buffer: buf.! !

!TSoundFrame methodsFor: 'loading' stamp: 'pbm 3/15/2006 14:07'!
loadSound: sound "a SampledSound"

	| data samplingRate format buf |

	(sound isKindOf: SampledSound) ifFalse: [ ^ self ].

	data := sound samples.
	samplingRate := sound originalSamplingRate.
	sound isStereo ifTrue: [ format := ALFormatStereo16 ]
		ifFalse: [ format := ALFormatMono16 ].

	buf := TSoundBuffer data: data samplingRate: samplingRate format: format.
	source buffer: buf.! !

!TSoundFrame methodsFor: 'loading' stamp: 'pbm 2/17/2006 17:19'!
loadWavFile: fileName
	
	| sound |
	sound := SampledSound fromWaveStream: (FileStream oldFileNamed: fileName).
	self loadSound: sound.! !

!TSoundFrame methodsFor: 'accessing' stamp: 'pbm 3/23/2006 13:28'!
looping: aBool
	source looping: aBool.! !

!TSoundFrame methodsFor: 'controls' stamp: 'pbm 3/23/2006 13:32'!
pause
	source pause.! !

!TSoundFrame methodsFor: 'controls' stamp: 'pbm 3/23/2006 13:32'!
play
	source play.! !

!TSoundFrame methodsFor: 'render' stamp: 'pbm 4/18/2006 15:35'!
renderFrame: ogl

	" apply the local transform to this matrix "
	frameChanged ifTrue:[
self islandViolation:'this code must be moved somewhere else'.
		self globalTransform: (frameParent globalTransform * localTransform).
	].

	(self root objectID) = ((ogl harness avatar get: #currentSpace) myName) ifTrue: [
		source render: ogl harness openAL.
	].



	^ 0! !

!TSoundFrame methodsFor: 'accessing' stamp: 'pbm 2/17/2006 17:19'!
source
	^ source! !

!TSoundFrame methodsFor: 'controls' stamp: 'pbm 3/23/2006 13:32'!
stop
	source stop.! !

Object subclass: #TVoiceChat
	instanceVariableNames: 'source sounds buffers bufferCount playing samplingRate'
	classVariableNames: ''
	poolDictionaries: 'OpenALConstants'
	category: 'OpenAL-Chat'!

!TVoiceChat commentStamp: '<historical>' prior: 0!
Manages sounds in a TSoundQueue for streaming onto a TSoundSource.

*** Make sure you end your voice chat before you quit Croquet. We don't handle this...yet. Also, a new participant cannot join a space while a voice chat is happening (for now). Wait until all participants are connected before starting a chat.

*** For now it doesn't do anything with the TSoundSource other than to get it's name in OpenAL to use the OpenAL object directly. This might change in the future.!

!TVoiceChat methodsFor: 'accessing' stamp: 'pbm 6/30/2006 17:20'!
addSoundData: data at: frameTime codec: codecClass
	playing ifTrue: [
		sounds addSoundData: data at: frameTime codec: codecClass.
	].! !

!TVoiceChat methodsFor: 'initialize' stamp: 'pbm 3/23/2006 10:21'!
initialize
	super initialize.

	source := TSoundSource new.
	sounds := TSoundQueue new.
	playing := false.
	samplingRate := 11025.
	bufferCount := 4.! !

!TVoiceChat methodsFor: 'buffering' stamp: 'pbm 5/16/2006 16:33'!
initializeBuffers: openAL

	| sourceID data |
	sourceID := (source nameIn: openAL) alID.
	buffers := openAL alGenBuffers: bufferCount.
	data := ByteArray new.

	"This is a hack to fill up some empty buffers to process and get the ball rolling."
	buffers do: [:bufferID |
		openAL alBufferData: bufferID
			with: ALFormatMono16
			with: data
			with: 0
			with: samplingRate.
	].

	openAL alSourceQueueBuffers: sourceID with: bufferCount with: buffers.
	openAL alSourcePlay: sourceID.

"queued := openAL alGetSourcei: sourceID with: ALBuffersQueued.
Transcript show: '****QUEUED**** = ', queued;cr."! !

!TVoiceChat methodsFor: 'controls' stamp: 'pbm 3/22/2006 17:07'!
play: openAL

	| sourceID |
	playing ifFalse: [
		playing := true.
		self initializeBuffers: openAL.
		self processBuffers: openAL.

		sourceID := (source nameIn: openAL) alID.
		self runScript: #processBuffers: withArguments: {openAL} when: {sounds. #newSound}
	].! !

!TVoiceChat methodsFor: 'buffering' stamp: 'pbm 3/23/2006 10:22'!
processBuffers: openAL

	| processed sourceID |

	playing ifTrue: [
		sourceID := (source nameIn: openAL) alID.
		processed := openAL alGetSourcei: sourceID with: ALBuffersProcessed.
"Transcript show: '****PROCESSED**** = ', processed;cr."
		processed timesRepeat: [ self rotateBuffer: openAL ].

		(openAL isPlaying: sourceID) ifFalse: [
			openAL alSourcePlay: sourceID.
		].
	].! !

!TVoiceChat methodsFor: 'buffering' stamp: 'pbm 3/31/2006 14:15'!
rotateBuffer: openAL

	| sourceID nextSound bufferID |
"Transcript show: sounds size;cr."

	sounds isEmpty ifFalse: [
		sourceID := (source nameIn: openAL) alID.

		nextSound := sounds nextSound.
		bufferID := openAL alSourceUnqueueBuffer: sourceID.
"Transcript show: 'bufferID = ', bufferID;cr."

		openAL alBufferData: bufferID
			with: ALFormatMono16
			with: nextSound
			with: nextSound byteSize
			with: samplingRate.

		openAL alSourceQueueBuffers: sourceID
			with: 1
			with: (WordArray with: bufferID).
	].! !

!TVoiceChat methodsFor: 'accessing' stamp: 'pbm 3/22/2006 14:41'!
sounds
	^ sounds! !

!TVoiceChat methodsFor: 'accessing' stamp: 'pbm 3/22/2006 14:41'!
source
	^ source! !

!TVoiceChat methodsFor: 'controls' stamp: 'pbm 3/23/2006 10:23'!
stop: openAL

	| sourceID bufs queued |
	playing ifTrue: [
		playing := false.
		sounds flush.
	
		sourceID := (source nameIn: openAL) alID.
		openAL alSourceStop: sourceID.
		queued := openAL alGetSourcei: sourceID with: ALBuffersQueued.
		bufs := openAL alSourceUnqueueBuffers: sourceID with: queued.
		openAL alDeleteBuffers: bufferCount with: bufs.
	].! !

!ExternalObject methodsFor: '*OpenAL' stamp: 'jcg 7/25/2006 01:23'!
isNull
	handle ifNil: [^ true].
	^ handle allSatisfy: [:byte | byte == 0]! !
AlcConstants initialize!
OpenAL initialize!
OpenALConstants initialize!
